@@ -16,12 +16,13 @@ from distutils.version import StrictVersion
 import json
 from glob import glob
 import tarfile
+import zipfile
 import uuid
 from enum import Enum
 
 from snakemake.exceptions import CreateCondaEnvironmentException, WorkflowError
 from snakemake.logging import logger
-from snakemake.common import strip_prefix
+from snakemake.common import strip_prefix, ON_WINDOWS
 from snakemake import utils
 from snakemake.deployment import singularity
 from snakemake.io import git_content
@@ -38,7 +39,7 @@ class CondaCleanupMode(Enum):
 def content(env_file):
     if env_file.startswith("git+file:"):
         return git_content(env_file).encode("utf-8")
-    elif urlparse(env_file).scheme:
+    elif urlparse(env_file).netloc:
         try:
             return urlopen(env_file).read()
         except URLError as e:
@@ -155,14 +156,13 @@ class Env:
                 out = shell.check_output(
                     "conda list --explicit --prefix '{}'".format(self.path),
                     stderr=subprocess.STDOUT,
+                    universal_newlines=True,
                 )
-                logger.debug(out.decode())
+                logger.debug(out)
             except subprocess.CalledProcessError as e:
-                raise WorkflowError(
-                    "Error exporting conda packages:\n" + e.output.decode()
-                )
+                raise WorkflowError("Error exporting conda packages:\n" + e.output)
             with open(os.path.join(env_archive, "packages.txt"), "w") as pkg_list:
-                for l in out.decode().split("\n"):
+                for l in out.split("\n"):
                     if l and not l.startswith("#") and not l.startswith("@"):
                         pkg_url = l
                         logger.info(pkg_url)
@@ -177,10 +177,13 @@ class Env:
                             r.raise_for_status()
                             copy.write(r.content)
                         try:
-                            tarfile.open(pkg_path)
+                            if pkg_path.endswith(".conda"):
+                                assert zipfile.ZipFile(pkg_path).testzip() is None
+                            else:
+                                tarfile.open(pkg_path)
                         except:
                             raise WorkflowError(
-                                "Package is invalid tar archive: {}".format(pkg_url)
+                                "Package is invalid tar/zip archive: {}".format(pkg_url)
                             )
         except (
             requests.exceptions.ChunkedEncodingError,
@@ -280,7 +283,9 @@ class Env:
                             cmd,
                             envvars=self.get_singularity_envvars(),
                         )
-                    out = shell.check_output(cmd, stderr=subprocess.STDOUT)
+                    out = shell.check_output(
+                        cmd, stderr=subprocess.STDOUT, universal_newlines=True
+                    )
 
                 else:
                     # Copy env file to env_path (because they can be on
@@ -306,7 +311,9 @@ class Env:
                             cmd,
                             envvars=self.get_singularity_envvars(),
                         )
-                    out = shell.check_output(cmd, stderr=subprocess.STDOUT)
+                    out = shell.check_output(
+                        cmd, stderr=subprocess.STDOUT, universal_newlines=True
+                    )
 
                     # cleanup if requested
                     if self._cleanup is CondaCleanupMode.tarballs:
@@ -321,7 +328,7 @@ class Env:
                 with open(os.path.join(env_path, "env_setup_done"), "a") as f:
                     pass
 
-                logger.debug(out.decode())
+                logger.debug(out)
                 logger.info(
                     "Environment for {} created (location: {})".format(
                         os.path.relpath(env_file), os.path.relpath(env_path)
@@ -332,7 +339,7 @@ class Env:
                 shutil.rmtree(env_path, ignore_errors=True)
                 raise CreateCondaEnvironmentException(
                     "Could not create conda environment from {}:\n".format(env_file)
-                    + e.output.decode()
+                    + e.output
                 )
 
         if tmp_file:
@@ -385,10 +392,15 @@ class Conda:
     def _check(self):
         from snakemake.shell import shell
 
+        # Use type here since conda now is a function.
+        # type allows to check for both functions and regular commands.
+        if not ON_WINDOWS or shell.get_executable():
+            locate_cmd = "type conda"
+        else:
+            locate_cmd = "where conda"
+
         try:
-            # Use type here since conda now is a function.
-            # type allows to check for both functions and regular commands.
-            shell.check_output(self._get_cmd("type conda"), stderr=subprocess.STDOUT)
+            shell.check_output(self._get_cmd(locate_cmd), stderr=subprocess.STDOUT)
         except subprocess.CalledProcessError as e:
             if self.container_img:
                 raise CreateCondaEnvironmentException(
@@ -418,8 +430,10 @@ class Conda:
                 )
         try:
             version = shell.check_output(
-                self._get_cmd("conda --version"), stderr=subprocess.STDOUT
-            ).decode()
+                self._get_cmd("conda --version"),
+                stderr=subprocess.STDOUT,
+                universal_newlines=True,
+            )
             version = version.split()[1]
             if StrictVersion(version) < StrictVersion("4.2"):
                 raise CreateCondaEnvironmentException(
