@@ -11,6 +11,7 @@ import struct
 from snakemake.remote import AbstractRemoteObject, AbstractRemoteProvider
 from snakemake.exceptions import WorkflowError, CheckSumMismatchException
 from snakemake.common import lazy_property
+import snakemake.io
 from snakemake.utils import os_sync
 
 try:
@@ -126,6 +127,29 @@ class RemoteObject(AbstractRemoteObject):
         self._bucket = None
         self._blob = None
 
+    def inventory(self, cache: snakemake.io.IOCache):
+        """Using client.list_blobs(), we want to iterate over the objects in
+           the "folder" of a bucket and store information about the IOFiles in the
+           provided cache (snakemake.io.IOCache) indexed by bucket/blob name.
+           This will be called by the first mention of a remote object, and
+           iterate over the entire bucket once (and then not need to again). 
+           This includes:
+            - cache.exist_remote
+            - cache_mtime
+            - cache.size
+        """
+        for blob in self.client.list_blobs(
+            self.bucket_name, prefix=os.path.dirname(self.blob.name)
+        ):
+            # By way of being listed, it exists. mtime is a datetime object
+            name = "{}/{}".format(blob.bucket.name, blob.name)
+            cache.exists_remote[name] = True
+            cache.mtime[name] = blob.updated
+            cache.size[name] = blob.size
+        # Mark bucket as having an inventory, such that this method is
+        # only called once for this bucket.
+        cache.has_inventory.add(self.bucket_name)
+
     # === Implementations of abstract class members ===
 
     @retry.Retry(predicate=google_cloud_retry_predicate)
